"""Handles all the citations and acknowledgements for tools built for Pandora"""

CITATIONS = {
    "pandorasat1": """@ARTICLE{2021arXiv210806438Q,
       author = {{Quintana}, Elisa V. and {Col{\'o}n}, Knicole D. and {Mosby}, Gregory and {Schlieder}, Joshua E. and {Supsinskas}, Pete and {Karburn}, Jordan and {Dotson}, Jessie L. and {Greene}, Thomas P. and {Hedges}, Christina and {Apai}, D{\'a}niel and {Barclay}, Thomas and {Christiansen}, Jessie L. and {Espinoza}, N{\'e}stor and {Mullally}, Susan E. and {Gilbert}, Emily A. and {Hoffman}, Kelsey and {Kostov}, Veselin B. and {Lewis}, Nikole K. and {Foote}, Trevor O. and {Mason}, James and {Youngblood}, Allison and {Morris}, Brett M. and {Newton}, Elisabeth R. and {Pepper}, Joshua and {Rackham}, Benjamin V. and {Rowe}, Jason F. and {Stevenson}, Kevin},
        title = "{The Pandora SmallSat: Multiwavelength Characterization of Exoplanets and their Host Stars}",
      journal = {arXiv e-prints},
     keywords = {Astrophysics - Instrumentation and Methods for Astrophysics, Astrophysics - Earth and Planetary Astrophysics},
         year = 2021,
        month = aug,
          eid = {arXiv:2108.06438},
        pages = {arXiv:2108.06438},
archivePrefix = {arXiv},
       eprint = {2108.06438},
 primaryClass = {astro-ph.IM},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2021arXiv210806438Q},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}""",
    "svofilters": """@INPROCEEDINGS{2020sea..confE.182R,
       author = {{Rodrigo}, C. and {Solano}, E.},
        title = "{The SVO Filter Profile Service}",
    booktitle = {XIV.0 Scientific Meeting (virtual) of the Spanish Astronomical Society},
         year = 2020,
        month = jul,
          eid = {182},
        pages = {182},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2020sea..confE.182R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
""",
}

ACKNOWLEDGEMENTS = {
    "svofilters": "This research has made use of the Spanish Virtual Observatory (https://svo.cab.inta-csic.es) project funded by MCIN/AEI/10.13039/501100011033/ through grant PID2020-112949GB-I00"
}
